(function(){
	'use strict';

	angular
		.module('app.auth')
		
		/**
		* AuthService is going to handle all of our auth functions, so we don't need to write them inside the controllers.
		*/
		.factory('AuthService', AuthService);

	function AuthService($q, $rootScope, $firebaseObject, $firebaseArray, $state, $firebaseAuth){

		return {
			/*
				The function receives an email, password, name and creates a new user
				After the user is created it stores the user details in the DB.
			*/
			signupEmail: function(newEmail, newPassword, newFullName){
				var deferred = $q.defer();

				firebase.auth().createUserWithEmailAndPassword(newEmail, newPassword).then(function() {
					// Sign-In successful.
					//console.log("Signup successful");

					var user = firebase.auth().currentUser;

					user.sendEmailVerification().then(function(result) { console.log(result) },function(error){ console.log(error)}); 

					user.updateProfile({
						displayName: newFullName,
					}).then(function() {
						// Update successful.
					}, function(error) {
						// An error happened.
						console.log(error);
					});

					deferred.resolve({
						status: true, 
						user: user
					});

				}, function(error) {
					deferred.resolve({
						status: false, 
						message: error
					});

				});

				return deferred.promise;
			},

			/**
			* Here we are login our user in, we user angular-fire $authWithPassword assing the email and password.
			* After that we send the user to our dashboard.
			*/
			loginUser: function(email, password){
				var deferred = $q.defer();

				firebase.auth().signInWithEmailAndPassword(email, password).then(function() {
					// Sign-In successful.
					
					var user = firebase.auth().currentUser;
					deferred.resolve({
						status: true, 
						user: user
					});

		        }, function(error) {
					deferred.resolve({
						status: false, 
						message: error
					});
		        });

				return deferred.promise;
			},

			logoutUser: function(){
				var deferred = $q.defer();

				firebase.auth().signOut().then(function() {
					deferred.resolve({
						status: true
					});

				}, function(error) {
					// An error happened.
					console.log(error);

					deferred.resolve({
						status: false
					});
				});

				return deferred.promise;
			},

			/**
			* This one explain itself, if the user doesn't remember his password he'll click in the "forgot you password?"
			* link and we need to send him a token so he can log in again
			*
			* NOTE: This doesn't send a reset password link, this sends a token he can use as a password to log in and
			* change his password to something he remembers.
			*/
			resetPassword: function(resetEmail){
				var deferred = $q.defer();

				firebase.auth().sendPasswordResetEmail(resetEmail).then(function(){
					deferred.resolve({
						status: true, 
						message: 'Reset password email sent. Check your inbox and login with new password'
					});
				}, function(error){
					deferred.resolve({
						status: false, 
						message: error
					});
				});

				return deferred.promise;
			},

			/**
			 * This is going to take the user's email + oldPassword + newPassword and change the Password
			 * used for login.
			 * After changing the password it's going to redirect the user to the profile page.
			 */
			changePassword: function(email, oldPassword, newPassword){
				authUser.$changePassword({
					email: email,
					oldPassword: oldPassword,
					newPassword: newPassword
				}).then(function(){
					alert('Password Changed!');
					$state.go('profile');
				}).catch(function(error){
					console.log(error);
				});
			},

			/**
			 * This is going to take the user's oldEmail + newEmail + password and change the email
			 * used for login.
			 *
			 * Remember, afeter changing the login email we also need to update the userProfile.userId node
			 * I'm going to handle this in the controller using the userProfileData function below.
			 */
			changeEmail: function(oldEmail, newEmail, password){
				authUser.$changeEmail({
					oldEmail: oldEmail,
					newEmail: newEmail,
					password: password
				}).then(function(){
						alert('Cambiaste tu correo!');
						$state.go('profile');
				}).catch(function(error){
					console.log(error);
				});
			},
			/**
			 * This will return the userProfile.userId node so we can update the email.
			 */
			userProfileData: function(userId){
				var userProfileRef = $firebaseRef.default.child('userProfile').child(userId);
				return $firebaseObject(userProfileRef);
			},
			/**
			 * This will return the logged in state of current user
			 */
			isLoggedin: function(){
				var retval = firebase.auth();
				console.log(retval);
				console.log($firebaseAuth);
				
				if(retval.currentUser != null){
					return true;
				}else{
					return false;
				}
			}

		}

	}
})();
