// Ionic Event Starter App
angular.module('starter', ['ionic', 'ngCordova', 'ngStorage', 'ngCordova.plugins', 'starter.controllers', 'starter.services', 'underscore', 'app.auth', 'app.events', 'firebase', 'auth.configs', 'ngNotify', 'angular-md5'])

.run(function($ionicPlatform, $window, $cordovaLocalNotification, $state, $rootScope, $localStorage, CONFIG, AuthService, ngNotify, $firebaseAuth, Event) {
    // console.log($localStorage);
    $ionicPlatform.ready(function() {

        // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
        // for form inputs)
        if ($window.cordova) {
            if ($window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }

            if ($window.cordova.plugins.notification && $window.cordova.plugins.notification.local) {
                // first remove all notification
                $cordovaLocalNotification.cancelAll().then(function(result) {
                    console.log('remove all notifications', result);
                });

                console.log('notifications enabled', $localStorage.myScheduleNotif);

                // Check login state
                firebase.auth().onAuthStateChanged(function(firebaseUser) {
                    if (firebaseUser) {
                        var uid = firebaseUser.uid;

                        if ($localStorage.myScheduleNotif) {
                            var notifications = [];
                            var mySchedule = $localStorage.mySchedule;
                            var myScheduleList = Event.getMySchedule(uid);

                            myScheduleList.then(function(response) {
                                angular.forEach(response, function(value, key) {
                                    var notifTime = moment(value.date, 'MM/DD/YYYY h:mm A').subtract(15, 'm').toDate();
                                    notifications.push({
                                        id: value.id,
                                        title: 'Event Reminder: ' + value.title + ' starts in 15 minutes!',
                                        text: value.date + ' ' + value.location,
                                        at: notifTime
                                    });
                                });

                                if (notifications.length > 0) {
                                    $cordovaLocalNotification.schedule(notifications).then(function(result) {
                                        console.log('add all notifications', result, $cordovaLocalNotification.getAll());
                                    });

                                    $rootScope.$on('$cordovaLocalNotification:click', function(event, notif, state) {
                                        // console.log(event, notif, state);
                                        $state.go('app.talk', {
                                            talkId: notif.id
                                        });
                                    });
                                }
                            });
                        }
                    }
                });


            }
        }

        if ($window.StatusBar) {
            // org.apache.cordova.statusbar required
            StatusBar.styleDefault();
        }

        //stateChange event
        $rootScope.$on("$stateChangeStart", function(event, toState, toParams, fromState, fromParams) {

            if (toState.views.menuContent.authRequired && !AuthService.isLoggedin()) {
                if (toState.url == '/schedule/:type' && toParams.type == 'my') {
                    $state.transitionTo("app.login");
                    event.preventDefault();
                    ngNotify.set('You have to login to access this feature', 'error');
                }
            }

        });

    });
})

.constant("moment", moment)

.constant('EURL', 'http://www.ubudwritersfestival.com/wp-json/wp/v2')

.config(function($stateProvider, $urlRouterProvider) {
    $stateProvider

        .state('app', {
        url: "/app",
        abstract: true,
        templateUrl: "templates/menu.html",
        controller: 'AppCtrl'
    })

    .state('app.home', {
        url: "/home",
        views: {
            'menuContent': {
                templateUrl: "templates/home.html",
                controller: 'HomeCtrl'
            }
        }
    })

    .state('app.how', {
        url: "/how-to-use",
        views: {
            'menuContent': {
                templateUrl: "templates/how-to-use.html",
                controller: 'HowToUseCtrl'
            }
        }
    })

    .state('app.login', {
        url: "/login",
        views: {
            'menuContent': {
                templateUrl: "app/auth/login.html",
                controller: 'LoginCtrl'
            }
        }
    })

    .state('app.forgot', {
        url: "/forgot-password",
        views: {
            'menuContent': {
                templateUrl: "app/auth/forgot-password.html",
                controller: 'ForgotCtrl'
            }
        }
    })

    .state('app.createaccount', {
        url: "/create-account",
        views: {
            'menuContent': {
                templateUrl: "app/auth/create-account.html",
                controller: 'CreateAccountCtrl'
            }
        }
    })

    .state('app.about', {
        url: "/about",
        views: {
            'menuContent': {
                templateUrl: "templates/about.html",
                controller: 'AboutCtrl'
            }
        }
    })

    .state('app.schedule', {
        url: "/schedule/:type",
        views: {
            'menuContent': {
                templateUrl: "templates/schedule.html",
                controller: 'ScheduleCtrl',
                authRequired: true
            }
        }
    })

    .state('app.talk', {
        url: "/talk/:talkId",
        views: {
            'menuContent': {
                templateUrl: "app/events/event.html",
                controller: 'TalkCtrl'
            }
        }
    })

    .state('app.speakers', {
        url: "/speakers",
        views: {
            'menuContent': {
                templateUrl: "templates/speakers.html",
                controller: 'SpeakersCtrl'
            }
        }
    })

    .state('app.search', {
        url: "/search",
        views: {
            'menuContent': {
                templateUrl: "templates/search.html",
                controller: 'SearchCtrl'
            }
        }
    })

    .state('app.speaker-detail', {
        url: '/speakers/:speakerId',
        views: {
            'menuContent': {
                templateUrl: 'templates/speaker-detail.html',
                controller: 'SpeakerDetailCtrl'
            }
        }
    })

    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/app/home');
});