angular.module('starter.controllers', [])

.controller('AppCtrl', function($scope, $rootScope, $state, Event, AuthService, ngNotify, CONFIG) {
    Event.fetch().then(function() {
        $scope.event = Event.get();
    });

    $scope.scheduleDaySelected = null;
    $scope.user = null;
    $scope.isLoggedin = null;

    firebase.auth().onAuthStateChanged(function(firebaseUser) {
        if (firebaseUser) {
            console.log("Signed in as:", firebaseUser.uid);
            $scope.user = firebaseUser;
            $scope.isLoggedin = true;
        } else {
            console.log("Signed out");
            $scope.user = null;
            $scope.isLoggedin = false;
        }
    });

    $scope.logout = function() {
        AuthService.logoutUser().then(function(retval) {
            console.log(retval);

            if (retval.status) {
                $rootScope.user = null;

                $state.go("app.home");
                ngNotify.set('Logout succeed!', 'success');
            } else {
                ngNotify.set('Logout failed!', 'error');
            }
        });
    };
})

//About: Shows the location in google maps,
//along with the location's wifi information and the event's twitter hashtag
.controller('AboutCtrl', function($scope, Event, $compile) {
    //fetch the event to display details
    //about the location, wifi and social media
    Event.fetch().then(function() {
        $scope.event = Event.get();
        initialize($scope.event.address);
    });

    //initialize google maps at element #map
    //and center at the given address
    function initialize(address) {
        geocoder = new google.maps.Geocoder();
        geocoder.geocode({
            'address': address
        }, function(results, status) {
            var location = results[0].geometry.location;
            if (status == google.maps.GeocoderStatus.OK) {
                var mapOptions = {
                    center: location,
                    zoom: 14,
                    mapTypeId: google.maps.MapTypeId.ROADMAP
                };
                var map = new google.maps.Map(document.getElementById("map"),
                    mapOptions);

                var marker = new google.maps.Marker({
                    position: location,
                    map: map
                });
            }

        });

        $scope.map = map;
    }

})

//Schedule: Displays the complete schedule.
//The talks are grouped by date. Talks can be added to My Schedule.
.controller('ScheduleCtrl', function($scope, Event, $stateParams, $location) {
    //This controller is used for the complete schedule if type is set to "all"
    //Otherwise, it displays the user's My Schedule
    var scheduleType = $stateParams.type;

    $scope.talkCategories = false;
    $scope.scheduleType = scheduleType;
    $scope.title = scheduleType == "all" ? "Schedule" : "My Schedule";
    $scope.showMySchedule = scheduleType != "all";
    $scope.myScheduleNotif = Event.getMyScheduleNotif();
    $scope.event = {
        days: [{
            id: 56,
            name: 26
        }, {
            id: 57,
            name: 27
        }, {
            id: 58,
            name: 28
        }, {
            id: 59,
            name: 29
        }, {
            id: 60,
            name: 30
        }, ],
        talks: [],
        talksTotalPages: 1,
        selectedDay: 56,
        month: 'October 2016'
    };

    function init() {
        // show the spinner
        $scope.processing = true;

        if ($scope.$root.scheduleDaySelected) {
            $scope.event.selectedDay = $scope.$root.scheduleDaySelected;
        }

        if ($scope.showMySchedule) {
            firebase.auth().onAuthStateChanged(function(firebaseUser) {
                if (firebaseUser) {
                    fetchMyTalks(firebaseUser.uid);
                }
            });
        } else {
            Event.fetchCategories()
                .then(function successCallback(response) {
                    $scope.talkCategories = response;
                    fetchTalks();
                    console.log(response);
                }, function errorCallback(response) {
                    init();
                    console.log(response);
                });
        }
    }

    init();

    //returns the days in an array given the date
    function getDays(date) {
        var days = new Array();
        var currentDate = new Date(date.from);
        var toDate = new Date(date.to);

        while (currentDate <= toDate) {
            days.push(currentDate.getDate());
            currentDate = new Date(currentDate.setDate(currentDate.getDate() + 1));
        }
        return days;
    }

    //show events from selected day
    $scope.selectDay = function(day) {
        $scope.processing = true;
        $scope.event.talks = [];
        $scope.event.selectedDay = day.id;
        $scope.$root.scheduleDaySelected = day.id;

        if ($scope.showMySchedule) {
            firebase.auth().onAuthStateChanged(function(firebaseUser) {
                if (firebaseUser) {
                    fetchMyTalks(firebaseUser.uid);
                }
            });
        } else {
            fetchTalks();
        }
    }

    $scope.showTalk = function(talkId) {
        $location.path("/app/talk/" + talkId);
    };

    //add event to my schedule
    $scope.toggleMySchedule = function(talkId) {
        Event.toggleMySchedule(talkId);
        $scope.selectDay($scope.event.selectedDay);
    }

    //add event to my schedule
    $scope.toggleMyScheduleNotif = function() {
        Event.toggleMyScheduleNotif();
    }

    $scope.getSpeakersById = function(id) {
        var speakers = Event.getSpeakerById(id);
        var names = speakers.map(function(elem) {
            return elem.name;
        }).join(", ");
        return names;
    }

    $scope.loadMore = function() {
        fetchTalks(true);
    };

    function fetchMyTalks(uid) {
        // Get my schedule list per day
        Event.getMyScheduleByDay(uid, $scope.event.selectedDay).then(function(myScheduleResponse) {

            if (myScheduleResponse.length > 0) {
                angular.forEach(myScheduleResponse, function(value, key) {

                    // for infividual event, fetch latest info from API
                    // so that user will view updated value if it has been changed
                    Event.fetchTalkInfo(value.id).then(function(eventInfoResponse) {

                        // update event info
                        myScheduleResponse[key].date = eventInfoResponse.date;
                        myScheduleResponse[key].date_iso = moment(eventInfoResponse.date, 'MM/DD/YYYY h:mm A').format("YYYY-MM-DD HH:mm:ss");
                        myScheduleResponse[key].dayId = eventInfoResponse.dayId;
                        myScheduleResponse[key].location = eventInfoResponse.location;
                        myScheduleResponse[key].time = eventInfoResponse.time;
                        myScheduleResponse[key].title = eventInfoResponse.title;

                        // assign scope value
                        $scope.event.talks = myScheduleResponse;
                        $scope.processing = false;
                    });

                });
            } else {
                // no my schedule event in the day selected
                $scope.processing = false;
            }

        });
    }

    function fetchTalks(nextpage) {
        $scope.event.talksCurrentPage = nextpage ? $scope.event.talksCurrentPage + 1 : 1;

        if ($scope.event.talksCurrentPage <= $scope.event.talksTotalPages) {
            Event.fetchTalksByDay($scope.event.selectedDay, $scope.event.talksCurrentPage)
                .then(function successCallback(response) {
                    // this callback will be called asynchronously
                    // when the response is available
                    $scope.processing = false;
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    console.log(response);

                    var talks = response.data;
                    for (var i = 0; i < talks.length; i++) {
                        var talk = talks[i];
                        talk.categories = [];
                        for (var j = 0; j < talk['program-category'].length; j++) {
                            talk.categories.push($scope.talkCategories[talk['program-category'][j]]);
                        }
                        talk.categories = talk.categories.join(', ');
                        $scope.event.talks.push(talk);
                    }
                    console.log($scope.event.talks);
                    $scope.event.talksTotalPages = response.headers('x-wp-totalpages');
                }, function errorCallback(response) {
                    // called asynchronously if an error occurs
                    // or server returns response with an error status.
                    $scope.processing = false;
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    console.log(response);

                    $scope.event.talks = false;
                });
        } else {
            $scope.$broadcast('scroll.infiniteScrollComplete');
        }
    }
})

.controller('SpeakersCtrl', function($scope, Event) {
    //get all of the speakers
    // Event.fetch().then(function() {
    //     $scope.speakers = Event.getSpeakers();
    // });

    $scope.speakers = [];
    $scope.search = {
        query: null
    };
    $scope.total = null;
    $scope.current = 0;

    $scope.fetchSpeakers = function() {
        // show the spinner
        $scope.current = $scope.current + 1;
        $scope.processing = $scope.current == 1;

        console.log($scope.processing)

        if ($scope.total === null || $scope.current <= $scope.total) {
            Event.fetchSpeakers($scope.current, $scope.search.query)
                .then(function successCallback(response) {
                    // this callback will be called asynchronously
                    // when the response is available
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    console.log(response);

                    var items = response.data;
                    for (var i = 0; i < items.length; i++) {
                        $scope.speakers.push(items[i]);
                    }
                    $scope.total = response.headers('x-wp-totalpages');
                    $scope.processing = false;
                }, function errorCallback(response) {
                    // called asynchronously if an error occurs
                    // or server returns response with an error status.
                    $scope.processing = false;
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    console.log(response);
                });
        } else {
            $scope.$broadcast('scroll.infiniteScrollComplete');
        }
    };

    $scope.searchSpeakers = function() {
        $scope.speakers = [];
        $scope.total = null;
        $scope.current = 0;

        $scope.fetchSpeakers();
    };

    $scope.resetSpeakers = function() {
        $scope.speakers = [];
        $scope.search.query = null;
        $scope.total = null;
        $scope.current = 0;

        console.log($scope);

        $scope.fetchSpeakers();
    };

    // get initial data
    // $scope.fetchSpeakers();
})

.controller('SpeakerDetailCtrl', function($scope, Event, $stateParams, moment) {
    // $scope.speaker = Event.getSpeakerById([parseInt($stateParams.speakerId)])[0];

    // var talks = Event.getTalksBySpeaker(parseInt($stateParams.speakerId));
    // //format date
    // for (var i = 0; i < talks.length; i++) {
    //   var talk = talks[i];
    //   talk.fDate = moment(talk.date).format("h:mm A, dddd, MMM Do");
    // }
    // $scope.talks = talks;

    // $scope.mySchedule = Event.getMySchedule();

    // //add event to my schedule
    // $scope.toggleMySchedule = function(talkId) {
    //   Event.toggleMySchedule(talkId);
    // }
    $scope.processing = true;
    $scope.speaker = null;
    $scope.talks = false;

    Event.fetchSpeakerInfo(parseInt($stateParams.speakerId))
        .then(function successCallback(response) {
            // this callback will be called asynchronously
            // when the response is available
            console.log(response);

            $scope.speaker = response;

            Event.fetchTalksById(response.talks).then(function(response) {
                $scope.processing = false;
                $scope.talks = response;
            });
        }, function errorCallback(response) {
            // called asynchronously if an error occurs
            // or server returns response with an error status.
            $scope.processing = false;
            console.log(response);
        });

})

//Search:
.controller('SearchCtrl', function($scope, Event) {

})

//How To Use
.controller('HowToUseCtrl', function($scope, Event) {
    $scope.processing = true;
    $scope.content = '';

    Event.getHowToUse().then(function successCallback(response) {
        // this callback will be called asynchronously
        // when the response is available
        $scope.processing = false;
        $scope.content = response.data.content.rendered;
        console.log(response);
    }, function errorCallback(response) {
        // called asynchronously if an error occurs
        // or server returns response with an error status.
        $scope.processing = false;
        console.log(response);
    });
})

//Home:
.controller('HomeCtrl', function($scope, Event, AuthService) {
    console.log(AuthService.isLoggedin());
    $scope.isLoggedin = AuthService.isLoggedin();
});