angular.module('starter.services', ['ngResource', 'ngStorage'])

/**
 * A factory to create a re-usable My Schedule object
 * we pass in a user id (uid) and get back their synchronized data
 */
.factory('MySchedule', ['$firebaseObject',
    function($firebaseObject) {
        return function(uid) {
            // create a reference to the database node where we will store our data
            var ref = firebase.database().ref('myschedule/' + uid);

            // return it as a synchronized object
            return $firebaseObject(ref);
        }
    }
])

/**
 * A simple example service that returns some data.
 */
.factory('Event', ['$resource', '$q', '$http', '_', 'moment', '$localStorage', 'EURL', '$filter', 'MySchedule', 'ngNotify',
    function($resource, $q, $http, _, moment, $localStorage, EURL, $filter, MySchedule, ngNotify) {

        var event = {};
        var talks = [];
        var speakers = [];
        var categories = {};

        if ($localStorage.myScheduleNotif == undefined) {
            $localStorage.myScheduleNotif = true;
        }

        return {
            fetch: function() {
                return $resource(EURL, {}, {
                    get: {
                        method: 'GET'
                    }
                }).get().$promise.then(function(data) {

                });
            },
            get: function() {
                return event;
            },
            getTalks: function() {
                return talks;
            },
            getSpeakers: function() {
                if (speakers.length == 0) {
                    // Simple GET request example:
                    $http({
                        cache: true,
                        method: 'GET',
                        url: 'http://www.ubudwritersfestival.com/wp-json/wp/v2/writer?filter[posts_per_page]=30'
                    }).then(function successCallback(response) {
                        // console.log(response)
                        var collection = response.data;
                        for (var i = 0; i < collection.length; i++) {
                            speakers.push({
                                id: collection[i].id,
                                name: collection[i].title.rendered,
                                image: collection[i].better_featured_image.source_url,
                                description: collection[i].content.rendered,
                                nationality: collection[i].acf.nationality
                            })

                            var speakerTalks = collection[i].acf.related_page;
                            for (var j = 0; j < speakerTalks.length; j++) {
                                talks.push({
                                    id: speakerTalks[j].ID,
                                    date: speakerTalks[j].post_date,
                                    title: speakerTalks[j].post_title,
                                    image: '',
                                    description: speakerTalks[j].post_content,
                                    speakers: [collection[i].id],
                                    location: '',
                                })
                            }
                        }
                        // console.log(speakers)
                        // this callback will be called asynchronously
                        // when the response is available
                    }, function errorCallback(response) {
                        // called asynchronously if an error occurs
                        // or server returns response with an error status.
                    });
                }

                return speakers;
            },
            getMySchedule: function(uid) {
                var deferred = $q.defer();
                var myScheduleList = MySchedule(uid);

                myScheduleList.$loaded().then(function() {
                    deferred.resolve(myScheduleList);
                });

                return deferred.promise;
            },
            getMyScheduleNotif: function() {
                return $localStorage.myScheduleNotif;
            },
            getMyScheduleByDay: function(uid, day) {
                var deferred = $q.defer();
                var myScheduleList = MySchedule(uid);
                var myScheduleByDay = [];
                var sortedMyScheduleByDay = [];

                myScheduleList.$loaded().then(function() {

                    angular.forEach(myScheduleList, function(value, key) {
                        console.log(value.dayId[0]);
                        if (_.contains(value.dayId, day)) {
                            value.date_iso = moment(value.date, "MM/DD/YYYY h:mm A").format("YYYY-MM-DD HH:mm:ss");
                            myScheduleByDay.push(value);
                        }
                    });

                    sortedMyScheduleByDay = $filter('orderBy')(myScheduleByDay, 'date_iso');

                    deferred.resolve(sortedMyScheduleByDay);

                    console.log(myScheduleList);
                    console.log(myScheduleByDay);
                    console.log(sortedMyScheduleByDay);

                });

                return deferred.promise;
            },
            toggleMySchedule: function(talk, uid) {
                var fTalk = {
                    id: talk.id,
                    dayId: talk.dayId,
                    date: moment(talk.date, 'M/D/YYYY h:mm A').format("M/D/YYYY h:mm A"),
                    notif: moment(talk.date, 'M/D/YYYY h:mm A').subtract(15, 'minutes').toDate(),
                    time: talk.time,
                    title: talk.title,
                    location: talk.location,
                    categories: talk.categories
                }

                var myScheculeEvent = MySchedule(uid + '/' + fTalk.id);

                myScheculeEvent.$loaded().then(function() {
                    // already added to my schedule, so we want to remove it
                    if (myScheculeEvent.id) {
                        myScheculeEvent.$remove().then(function() {
                            ngNotify.set('You have removed this event from your schedule.', 'success');
                        }).catch(function(error) {
                            ngNotify.set('Oops, Something wrong while removing this event!.', 'error');
                        });

                        // not added yet, so we want to add
                    } else {
                        // Update or ctrate object
                        myScheculeEvent.id = fTalk.id;
                        myScheculeEvent.dayId = fTalk.dayId;
                        myScheculeEvent.date = fTalk.date;
                        myScheculeEvent.notif = fTalk.notif;
                        myScheculeEvent.time = fTalk.time;
                        myScheculeEvent.title = fTalk.title;
                        myScheculeEvent.location = fTalk.location;
                        myScheculeEvent.categories = fTalk.categories;

                        myScheculeEvent.$save().then(function afterAddMySchedule(resp) {
                            ngNotify.set('You have added this event to your schedule.', 'success');
                        }).catch(function(error) {
                            ngNotify.set('Oops, Something wrong while adding this event!.', 'error');
                        });
                    }
                });
            },
            toggleMyScheduleNotif: function() {
                $localStorage.myScheduleNotif = !$localStorage.myScheduleNotif;
                console.log($localStorage.myScheduleNotif);
            },
            getTalksByDay: function(day, talks) {
                //filter by date
                var filteredTalks = _.filter(talks, function(talk) {
                    return new Date(talk.date).getDate() == day;
                });
                //group by time
                filteredTalks = _.chain(_.groupBy(filteredTalks, function(talk) {
                    return moment(talk.date).format("h:mm A");
                })).pairs().sortBy(function(data) {
                    return new Date(data[0])
                }).value();
                return filteredTalks;
            },
            getTalksById: function(id) {
                var getById = function() {
                    //filter by id
                    var filteredTalks = _.filter(talks, function(talk) {
                        return talk.id == id;
                    });
                    if (filteredTalks.length == 1) {
                        return filteredTalks[0];
                    } else {
                        return {};
                    }
                }
                if (talks.length == 0) {
                    this.fetch().then(function() {
                        return getById();
                    });
                } else {
                    return getById();
                }
            },
            getTalksBySpeaker: function(id) {
                return _.filter(talks, function(talk) {
                    return _.contains(talk.speakers, id);
                });
            },
            getSpeakerById: function(idArr) {
                return _.filter(speakers, function(speaker) {
                    return _.contains(idArr, speaker.id);
                });
            },
            getHowToUse: function() {
                // Simple GET request example:
                return $http({
                    cache: true,
                    method: 'GET',
                    url: 'http://www.ubudwritersfestival.com/wp-json/wp/v2/pages/3803'
                });
            },
            fetchSpeakers: function(page, query) {
                page = page || 1;

                var deferred = $q.defer();
                var limit = 10;
                var params = {
                    'filter[posts_per_page]': limit,
                    page: page
                }

                if (query) {
                    params['filter[s]'] = query;
                }

                $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/writer',
                    params: params
                }).success(function(data, status, headers, config) {
                    var items = [];

                    for (var i = 0; i < data.length; i++) {
                        items.push({
                            id: data[i].id,
                            name: data[i].title.rendered,
                            image: data[i].better_featured_image.source_url,
                            description: data[i].content.rendered,
                            nationality: data[i].acf.nationality
                        })
                    }

                    deferred.resolve({
                        config: config,
                        data: items,
                        headers: headers,
                        status: status
                    });
                }).error(function(data, status, headers, config) {
                    deferred.reject({
                        config: config,
                        data: data,
                        headers: headers,
                        status: status
                    });
                });

                return deferred.promise;
            },
            fetchSpeakersById: function(ids) {
                var deferred = $q.defer();
                var limit = -1;

                $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/writer',
                    params: {
                        'filter[posts_per_page]': limit,
                        include: ids
                    }
                }).success(function(response) {
                    var items = [];

                    for (var i = 0; i < response.length; i++) {
                        items.push({
                            id: response[i].id,
                            name: response[i].title.rendered,
                            image: response[i].better_featured_image.source_url,
                            description: response[i].content.rendered,
                            nationality: response[i].acf.nationality
                        })
                    }

                    deferred.resolve(items);
                }).error(function(response) {
                    deferred.reject(response);
                });

                return deferred.promise;
            },
            fetchCategories: function() {
                var deferred = $q.defer();
                var limit = 30;

                $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/program-category',
                    params: {
                        'per_page': limit
                    }
                }).success(function(response) {
                    var items = {};

                    for (var i = 0; i < response.length; i++) {
                        items[response[i].id] = response[i].name;
                    }

                    categories = items;
                    deferred.resolve(items);
                }).error(function(response) {
                    deferred.reject(response);
                });

                return deferred.promise;
            },
            fetchSpeakerInfo: function(id) {
                var deferred = $q.defer();

                $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/writer/' + id
                }).success(function(response) {
                    var speakerInfo = response;
                    var speakerTalks = speakerInfo.acf.related_page;

                    var detail = {
                        id: speakerInfo.id,
                        name: speakerInfo.title.rendered,
                        image: speakerInfo.better_featured_image.source_url,
                        description: speakerInfo.content.rendered,
                        nationality: speakerInfo.acf.nationality,
                        talks: []
                    };

                    for (var i = 0; i < speakerTalks.length; i++) {
                        detail.talks.push(speakerTalks[i].ID);
                    }
                    detail.talks = detail.talks.join(',');

                    deferred.resolve(detail);
                }).error(function(response) {
                    deferred.reject(response);
                });

                return deferred.promise;
            },
            fetchTalkInfo: function(id) {
                var deferred = $q.defer();

                // Simple GET request example:
                $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/program/' + id
                }).success(function(response) {
                    var talkInfo = response;
                    var talkSpeakers = talkInfo.acf.related_page;
                    var talkCategories = talkInfo['program-category'];

                    var detail = {
                        id: talkInfo.id,
                        date: moment(talkInfo.acf.start_date + ' ' + talkInfo.acf.start_time, 'YYYYMMDD h:mm').format("M/D/YYYY h:mm A"),
                        time: talkInfo.acf.start_time + ' - ' + talkInfo.acf.end_time,
                        title: talkInfo.title.rendered,
                        location: talkInfo.acf.venue.trim() + '. ' + talkInfo.acf.address,
                        description: talkInfo.content.rendered,
                        dayId: talkInfo['program-day'],
                        link: talkInfo.link,
                        categories: [],
                        speakers: []
                    };

                    for (var i = 0; i < talkCategories.length; i++) {
                        if (categories[talkCategories[i]]) {
                            detail.categories.push(categories[talkCategories[i]])
                        }
                    }
                    detail.categories = detail.categories.join(', ');

                    for (var i = 0; i < talkSpeakers.length; i++) {
                        detail.speakers.push(talkSpeakers[i].ID);
                    }
                    detail.speakers = detail.speakers.join(',');

                    deferred.resolve(detail);
                }).error(function(response) {
                    deferred.reject(response);
                });

                return deferred.promise;
            },
            fetchTalksByDay: function(day, page) {
                // Simple GET request example:
                return $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/program',
                    params: {
                        'program-day': day,
                        page: page
                    }
                });
            },
            fetchTalksById: function(ids) {
                var deferred = $q.defer();
                var limit = -1;

                // Simple GET request example:
                $http({
                    cache: true,
                    method: 'GET',
                    url: EURL + '/program',
                    params: {
                        'filter[posts_per_page]': limit,
                        include: ids
                    }
                }).success(function(response) {
                    var items = [];

                    for (var i = 0; i < response.length; i++) {
                        var item = response[i];
                        items.push({
                            id: item.id,
                            date: moment(item.acf.start_date + ' ' + item.acf.start_time, 'YYYYMMDD h:mm').format("h:mm A, dddd, MMM Do"),
                            title: item.title.rendered,
                            location: item.acf.venue.trim() + '. ' + item.acf.address,
                            description: item.content.rendered,
                            link: item.link,
                            categories: [],
                            speakers: []
                        })
                    }

                    deferred.resolve(items);
                }).error(function(response) {
                    deferred.reject(response);
                });

                return deferred.promise;
            }
        }
    }
]);